@@ -12,6 +12,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import functools
 import logging
 import threading
 
@@ -19,7 +20,6 @@ import fasteners
 from oslo_utils import encodeutils
 from oslo_utils import timeutils
 import requests
-import six
 
 import tooz
 from tooz import coordination
@@ -33,7 +33,7 @@ LOG = logging.getLogger(__name__)
 def _translate_failures(func):
     """Translates common requests exceptions into tooz exceptions."""
 
-    @six.wraps(func)
+    @functools.wraps(func)
     def wrapper(*args, **kwargs):
         try:
             return func(*args, **kwargs)
@@ -106,11 +106,11 @@ class EtcdLock(locking.Lock):
         reply = self.client.delete(self._lock_url, make_url=False)
         return reply.get('errorCode') is None
 
-    def acquire(self, blocking=True, shared=False):
+    def acquire(self, blocking=True, shared=False, timeout=None):
         if shared:
             raise tooz.NotImplemented
 
-        blocking, timeout = utils.convert_blocking(blocking)
+        blocking, timeout = utils.convert_blocking(blocking, timeout)
         if timeout is not None:
             watch = timeutils.StopWatch(duration=timeout)
             watch.start()
