@@ -17,6 +17,7 @@
 from distutils import version
 import functools
 import logging
+import re
 import string
 import threading
 
@@ -34,7 +35,7 @@ from tooz import utils
 LOG = logging.getLogger(__name__)
 
 
-def _handle_failures(func=None, n_tries=15):
+def _handle_failures(n_tries=15):
 
     """Translates common redis exceptions into tooz exceptions.
 
@@ -43,37 +44,37 @@ def _handle_failures(func=None, n_tries=15):
     :param func: the function to act on
     :param n_tries: the number of retries
     """
+    def inner(func):
+        @functools.wraps(func)
+        def wrapper(*args, **kwargs):
+            ntries = n_tries
+            while ntries:
+                try:
+                    return func(*args, **kwargs)
+                except exceptions.ConnectionError as e:
+                    # retry ntries times and then raise a connection error
+                    ntries -= 1
+                    if not ntries:
+                        LOG.debug(
+                            "Redis connection error, "
+                            "retry limit has been reached, aborting - %s", e
+                        )
+                        utils.raise_with_cause(
+                            coordination.ToozConnectionError,
+                            encodeutils.exception_to_unicode(e),
+                            cause=e)
+                    LOG.debug("Redis connection error, will retry - %s", e)
 
-    if func is None:
-        return functools.partial(
-                _handle_failures,
-                n_tries=n_tries
-                )
-
-    @functools.wraps(func)
-    def wrapper(*args, **kwargs):
-        ntries = n_tries
-        while ntries > 1:
-            try:
-                return func(*args, **kwargs)
-            except exceptions.ConnectionError as e:
-                # retry ntries times and then raise a connection error
-                ntries -= 1
-                if ntries >= 1:
+                except (exceptions.TimeoutError) as e:
                     utils.raise_with_cause(coordination.ToozConnectionError,
                                            encodeutils.exception_to_unicode(e),
                                            cause=e)
-
-            except (exceptions.TimeoutError) as e:
-                utils.raise_with_cause(coordination.ToozConnectionError,
-                                       encodeutils.exception_to_unicode(e),
-                                       cause=e)
-            except exceptions.RedisError as e:
-                utils.raise_with_cause(tooz.ToozError,
-                                       encodeutils.exception_to_unicode(e),
-                                       cause=e)
-        return func(*args, **kwargs)
-    return wrapper
+                except exceptions.RedisError as e:
+                    utils.raise_with_cause(tooz.ToozError,
+                                           encodeutils.exception_to_unicode(e),
+                                           cause=e)
+        return wrapper
+    return inner
 
 
 class RedisLock(locking.Lock):
@@ -89,7 +90,7 @@ class RedisLock(locking.Lock):
         self._coord = coord
         self._client = client
 
-    @_handle_failures
+    @_handle_failures()
     def is_still_owner(self):
         lock_tok = self._lock.local.token
         if not lock_tok:
@@ -97,15 +98,15 @@ class RedisLock(locking.Lock):
         owner_tok = self._client.get(self.name)
         return owner_tok == lock_tok
 
-    @_handle_failures
+    @_handle_failures()
     def break_(self):
         return bool(self._client.delete(self.name))
 
-    @_handle_failures
-    def acquire(self, blocking=True, shared=False):
+    @_handle_failures()
+    def acquire(self, blocking=True, shared=False, timeout=None):
         if shared:
             raise tooz.NotImplemented
-        blocking, timeout = utils.convert_blocking(blocking)
+        blocking, timeout = utils.convert_blocking(blocking, timeout)
         acquired = self._lock.acquire(
             blocking=blocking, blocking_timeout=timeout)
         if acquired:
@@ -113,7 +114,7 @@ class RedisLock(locking.Lock):
                 self._coord._acquired_locks.add(self)
         return acquired
 
-    @_handle_failures
+    @_handle_failures()
     def release(self):
         with self._exclusive_access:
             try:
@@ -125,7 +126,7 @@ class RedisLock(locking.Lock):
                 self._coord._acquired_locks.discard(self)
             return True
 
-    @_handle_failures
+    @_handle_failures()
     def heartbeat(self):
         with self._exclusive_access:
             if self.acquired:
@@ -159,7 +160,8 @@ class RedisDriver(coordination.CoordinationDriverCachedRunWatchers,
 
     The Redis driver connection URI should look like::
 
-      redis://[:PASSWORD@]HOST:PORT[?OPTION=VALUE[&OPTION2=VALUE2[&...]]]
+      redis://[USERNAME:PASSWORD@]HOST:PORT
+          [?OPTION=VALUE[&OPTION2=VALUE2[&...]]]
 
     For a list of options recognized by this driver, see the documentation
     for the member CLIENT_ARGS, and to determine the expected types of those
@@ -257,9 +259,12 @@ class RedisDriver(coordination.CoordinationDriverCachedRunWatchers,
         'retry_on_timeout',
         'socket_keepalive',
         'socket_timeout',
+        'socket_connect_timeout',
+        'health_check_interval',
         'ssl',
         'ssl_certfile',
         'ssl_keyfile',
+        'ssl_ca_certs',
         'sentinel',
         'sentinel_fallback',
     ])
@@ -281,14 +286,20 @@ class RedisDriver(coordination.CoordinationDriverCachedRunWatchers,
     #: Client arguments that are expected to be boolean convertible.
     CLIENT_BOOL_ARGS = frozenset([
         'retry_on_timeout',
+        'socket_keepalive',
         'ssl',
     ])
 
     #: Client arguments that are expected to be int convertible.
     CLIENT_INT_ARGS = frozenset([
         'db',
-        'socket_keepalive',
+    ])
+
+    #: Client arguments that are expected to be float convertible.
+    CLIENT_FLOAT_ARGS = frozenset([
         'socket_timeout',
+        'socket_connect_timeout',
+        'health_check_interval',
     ])
 
     #: Default socket timeout to use when none is provided.
@@ -410,6 +421,18 @@ return 1
     _dumps = staticmethod(utils.dumps)
     _loads = staticmethod(utils.loads)
 
+    @classmethod
+    def _parse_sentinel(cls, sentinel):
+        # IPv6 (eg. [::1]:6379 )
+        match = re.search(r'^\[(\S+)\]:(\d+)$', sentinel)
+        if match:
+            return (match[1], int(match[2]))
+        # IPv4 or hostname (eg. 127.0.0.1:6379 or localhost:6379)
+        match = re.search(r'^(\S+):(\d+)$', sentinel)
+        if match:
+            return (match[1], int(match[2]))
+        raise ValueError('Malformed sentinel server format')
+
     @classmethod
     def _make_client(cls, parsed_url, options, default_socket_timeout):
         kwargs = {}
@@ -421,6 +444,8 @@ return 1
             if not parsed_url.path:
                 raise ValueError("Expected socket path in parsed urls path")
             kwargs['unix_socket_path'] = parsed_url.path
+        if parsed_url.username:
+            kwargs['username'] = parsed_url.username
         if parsed_url.password:
             kwargs['password'] = parsed_url.password
         for a in cls.CLIENT_ARGS:
@@ -432,6 +457,8 @@ return 1
                 v = options[a]
             elif a in cls.CLIENT_INT_ARGS:
                 v = int(options[a])
+            elif a in cls.CLIENT_FLOAT_ARGS:
+                v = float(options[a])
             else:
                 v = options[a]
             kwargs[a] = v
@@ -442,24 +469,22 @@ return 1
         # sentinel arg.
         if 'sentinel' in kwargs:
             sentinel_hosts = [
-                tuple(fallback.split(':'))
-                for fallback in kwargs.get('sentinel_fallback', [])
+                cls._parse_sentinel(fallback)
+                for fallback in kwargs.pop('sentinel_fallback', [])
             ]
-            sentinel_hosts.insert(0, (kwargs['host'], kwargs['port']))
+            sentinel_hosts.insert(0, (kwargs.pop('host'), kwargs.pop('port')))
+            sentinel_name = kwargs.pop('sentinel')
             sentinel_server = sentinel.Sentinel(
                 sentinel_hosts,
-                socket_timeout=kwargs['socket_timeout'])
-            sentinel_name = kwargs['sentinel']
-            del kwargs['sentinel']
-            if 'sentinel_fallback' in kwargs:
-                del kwargs['sentinel_fallback']
-            master_client = sentinel_server.master_for(sentinel_name, **kwargs)
-            # The master_client is a redis.StrictRedis using a
+                sentinel_kwargs=kwargs,
+                **kwargs)
+            master_client = sentinel_server.master_for(sentinel_name)
+            # The master_client is a redis.Redis using a
             # Sentinel managed connection pool.
             return master_client
-        return redis.StrictRedis(**kwargs)
+        return redis.Redis(**kwargs)
 
-    @_handle_failures
+    @_handle_failures()
     def _start(self):
         super(RedisDriver, self)._start()
         try:
@@ -532,7 +557,7 @@ return 1
     def _decode_group_id(self, group_id):
         return utils.to_binary(group_id, encoding=self._encoding)
 
-    @_handle_failures
+    @_handle_failures()
     def heartbeat(self):
         beat_id = self._encode_beat_id(self._member_id)
         expiry_ms = max(0, int(self.membership_timeout * 1000.0))
@@ -547,7 +572,7 @@ return 1
                             exc_info=True)
         return min(self.lock_timeout, self.membership_timeout)
 
-    @_handle_failures
+    @_handle_failures()
     def _stop(self):
         while self._acquired_locks:
             lock = self._acquired_locks.pop()
