@@ -15,8 +15,6 @@
 #    under the License.
 import abc
 
-import six
-
 import tooz
 from tooz import coordination
 
@@ -34,8 +32,7 @@ class _LockProxy(object):
         self.lock.__exit__(exc_type, exc_val, exc_tb)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class Lock(object):
+class Lock(object, metaclass=abc.ABCMeta):
     def __init__(self, name):
         if not name:
             raise ValueError("Locks must be provided a name")
@@ -97,13 +94,17 @@ class Lock(object):
         raise tooz.NotImplemented
 
     @abc.abstractmethod
-    def acquire(self, blocking=True):
+    def acquire(self, blocking=True, shared=False, timeout=None):
         """Attempts to acquire the lock.
 
         :param blocking: If True, blocks until the lock is acquired. If False,
                          returns right away. Otherwise, the value is used as a
                          timeout value and the call returns maximum after this
                          number of seconds.
+        :param shared: If False, the lock is exclusive. If True, the lock can
+                       be shareable or raises ``NotImplemented`` if not
+                       implemented.
+        :param timeout: Timeout to acquire a lock.
         :returns: returns true if acquired (false if not)
         :rtype: bool
         """
