@@ -16,10 +16,10 @@
 
 import threading
 import time
+from unittest import mock
+import urllib
 
 from concurrent import futures
-import mock
-from six.moves.urllib import parse
 from testtools import matchers
 from testtools import testcase
 
@@ -49,7 +49,7 @@ class TestAPI(tests.TestWithCoordinator):
         if (tooz.coordination.Characteristics.DISTRIBUTED_ACROSS_HOSTS
            not in self._coord.CHARACTERISTICS):
             self.skipTest("This driver is not distributed across hosts")
-        scheme = parse.urlparse(self.url).scheme
+        scheme = urllib.parse.urlparse(self.url).scheme
         coord = tooz.coordination.get_coordinator(
             "%s://localhost:1/f00" % scheme,
             self.member_id)
@@ -818,22 +818,10 @@ class TestAPI(tests.TestWithCoordinator):
             self.assertTrue(f.result())
 
     def test_get_lock_concurrency_locking_two_lock_process(self):
-        # NOTE(jd) Using gRPC and forking is not supported so this test might
-        # very likely hang forever or crash. See
-        # https://github.com/grpc/grpc/issues/10140#issuecomment-297548714 for
-        # more info.
-        if self.url.startswith("etcd3://"):
-            self.skipTest("Unable to use etcd3 with fork()")
         self._do_test_get_lock_concurrency_locking_two_lock(
             futures.ProcessPoolExecutor, False)
 
     def test_get_lock_serial_locking_two_lock_process(self):
-        # NOTE(jd) Using gRPC and forking is not supported so this test might
-        # very likely hang forever or crash. See
-        # https://github.com/grpc/grpc/issues/10140#issuecomment-297548714 for
-        # more info.
-        if self.url.startswith("etcd3://"):
-            self.skipTest("Unable to use etcd3 with fork()")
         self._do_test_get_lock_serial_locking_two_lock(
             futures.ProcessPoolExecutor, False)
 
